//==============================================================================
//	
//	Copyright (c) 2002-
//	Authors:
//	* Dave Parker <david.parker@comlab.ox.ac.uk> (University of Oxford)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

package pta;
import java.util.*;
/**
 * Implementation of the difference-bound matrix (DBM) data structure.
 * 
 * Data structures and algorithms are mostly based on those from: 
 * Johan Bengtsson, Wang Yi: Timed Automata: Semantics, Algorithms and Tools.
 * Lectures on Concurrency and Petri Nets 2003, LNCS volume 3098, pages 87-124, Springer, 2004
 */
public class DBM extends Zone
{
	/* Parent PTA */
	public PTA pta;
	/*
	 * Canonical zone representation: DBM
	 * (numClocks+1)^2 matrix d, indexed 1...numClocks for clocks in PTA and 0 for special zero clock.
	 * Each entry d[i][j] gives the bound for clock difference xi-xj.
	 * Difference bounds are encoded as a single integer; see help class DB for details.
	 */
	protected int d[][];

	/**
	 * Construct an empty DBM (don't use this).
	 */
	public DBM(PTA pta)
	{
		this.pta = pta;
		this.d = new int[pta.numClocks + 1][pta.numClocks + 1];
	}

	public DBM(PTA pta, int d[][])
	{
		this.pta = pta;
		this.d = d;
	}
	//splitting methods

	//returns wether the (upper if b, lower if not b) bound on x_i is useless 
	public boolean isUseless(int i, boolean b){
		PTA pta = this.getPTA();
		int n = pta.numClocks + 1;
		boolean acc = false;
		if (b){	
			for (int j = 1; j < n & !acc; j++){
				if (j != i) {acc = (acc || (this.d[i][0] >= DB.add(this.d[i][j], this.d[j][0])));}
			}

		}	

		else{	
			for (int j = 1; j < n & !acc; j++){
				if (j != i) {acc = (acc || (this.d[0][i] >= DB.add(this.d[0][j] ,this.d[j][i])));}
			}
		}	

		return acc;	
	}


	//remove useless constant constraints
	public void reduce(){
		this.canonicalise();
		PTA pta = this.getPTA();
		int n = pta.numClocks + 1;
		for(int i = 0; i < n; i++){
			if (this.isUseless(i,true)){
				this.d[i][0] = DB.INFTY;
				//System.out.printf("\n est inutile : x_%d, upper",i);
			}
			if (this.isUseless(i,false)){
				//System.out.printf("\n est inutile : x_%d, lower",i);
				this.d[0][i] = DB.INFTY;
			}
		}
	}

	//FAULTYchange the guard to empty if punctual and put the guard in reduced form.
	public void PunctualtoEmpty(){}
	//{
	//this.canonicalise();
	//int n = pta.numClocks + 1;
	//System.out.printf("\n00 %d\n",n);
	//for(int i = 1; i < n; i++){
	//System.out.printf("\ni:%d  bound:%d\n",i,d[i][0]);
	//	if(d[i][0]<=1){
	//System.out.printf("\n1 %s\n",this.toString());
	//		this.d[0][0]=-1;} 
	//	for(int j =i+1 ; j < n; j++){
	//		if(!DB.isInfty(d[i][j]) & DB.compareBounds(d[i][j],d[j][i])){
	//System.out.printf("\n %d %d %d %d %s\n",i,j,d[i][j],d[j][i],this.toString()); 
	//			this.d[0][0]=-1;};
	//	}
	//}
	//this.reduce();//NOT SURE TO KEEP THAT;
	//}

	//(!isInfty(d2) && (-getSignedDiff(d1) == getSignedDiff(d2))) 

	//return an equivalent (the union represents the same zone) list of DBM,
	//nicer for calculating volumes

	//public DBMandBound addBound(){
	//DBMandBound res = new DBMandBound(pta);
	//      LinkedList<Integer> upperbounds = new LinkedList();
	//      LinkedList<Integer> lowerbounds = new LinkedList();
	//      //this.printDBM();
	//      //this.printDBM();
	//      //System.out.print("\nmatrice reduite canonique" + this.toString() + "\n");
	//      this.reduce();
	//      //this.printDBM();
	//      //System.out.print("\nmatrice reduite" + this.toString() + "\n");
	//
	//      PTA pta = this.getPTA();
	//      int n = pta.numClocks + 1;
	//	//spliting regardings lower bounds
	//	for(int i = 1; i < n; i++){	
	//	  if (this.d[0][i] != DB.INFTY){
	//	   lowerbounds.add(i);
	//	    }
	//	  }
	//	  for(int i = 1; i < n; i++){	
	//	  if (this.d[i][0] != DB.INFTY){
	//	  upperbounds.add(i);
	//	    }
	//	  }
	//res.pta =this.pta;
	//res.d=this.d;
	//res.upperbounds=upperbounds;
	//res.lowerbounds=lowerbounds;
	////System.out.printf("\n %s \n", res.toString());
	//return(res);
	//}


	//public LinkedList<DBM> mySplit (){
	//DBMandBound  M= this.addBound();
	//return(); 
	//}


	//return an equivalent (the union represents the same zone) list of DBM,
	//nicer for calculating volumes
	//public LinkedList<DBM> mySplitvieux (){
	//      //System.out.printf("\n %s \n", this.toString());
	//      LinkedList<Integer> l = new LinkedList();
	//      LinkedList<DBM> res = new LinkedList();
	//      //this.printDBM();
	//      //this.printDBM();
	//      //System.out.print("\nmatrice reduite canonique" + this.toString() + "\n");
	//      this.reduce();
	//      //this.printDBM();
	//      //System.out.print("\nmatrice reduite" + this.toString() + "\n");
	//
	//      PTA pta = this.getPTA();
	//      int n = pta.numClocks + 1;
	//	//spliting regardings lower bounds
	//	for(int i = 1; i < n; i++){	
	//	  if (this.d[0][i] != DB.INFTY){
	//	   l.add(i);
	//	    }
	//	  }
	//	  int k = l.size();
	//	  //System.out.print("\nl:" + l.toString() + "\n");
	//	
	//	  for (int i = 0; i < k; i++){
	//	   int g = l.get(i);
	//	   DBM res_aux = this.deepCopy();
	//	    for(int j = 0; j < k; j++ ){
	//	    if (j != i){
	//	    	res_aux.setConstraint(0, l.get(j), 0);//useless
	//	    	res_aux.addConstraint(g,l.get(j), DB.add(this.d[0][l.get(j)], DB.opposite((this.d[0][g]))));
	//	    	}
	//	    }
	//	  res_aux.PunctualtoEmpty();//remove non punctual guard
	//	  if (!res_aux.isEmpty() & !res_aux.equals(this)) {res.add(res_aux);}
	//	  }
	//	  l.clear();
	//	 
	//	  //spliting regardings upper bounds
	//	  for(int i = 1; i < n; i++){
	//	  if (this.d[i][0] != DB.INFTY){
	//	  l.add(i);
	//	    }
	//	  }
	//	  int k2 = l.size();
	////System.out.print("\nlup:" + l.toString() + "\n");
	//	  for (int i = 0; i < k2; i++){
	//	   int g = l.get(i);
	//	   DBM res_aux = this.deepCopy();
	//	    for(int j = 0; j < k2; j++ ){
	//	      if (j != i){
	//	      //System.out.printf("\n l.get(j): %d,  res_aux: %s \n",l.get(j), res_aux.toString());
	//	      res_aux.setConstraint(l.get(j), 0, DB.INFTY);//useless
	//	      //System.out.printf("\n  res_aux: %s \n", res_aux.toString());
	//	      res_aux.addConstraint(l.get(j), g , DB.add((this.d[l.get(j)][0]), DB.opposite(this.d[g][0])));
	//	     // System.out.printf("\n  res_aux: %s \n", res_aux.toString());
	//	      }
	//	    }
	//	  	  res_aux.PunctualtoEmpty();//make punctual guard empty
	//	  if (!res_aux.isEmpty() & !res_aux.equals(this)) {res.add(res_aux);}//TO DO remove non punctual guard
	//	  //System.out.printf("\n l.get(i): %d,  res_aux: %s \n",l.get(i), res_aux.toString());
	//	  //res_aux.printDBM();
	//	  }
	//	int k3 = res.size();
	//	      
	//	//apply Floyd-Warshall in res
	//	for(int i = 0; i < k3; i++){
	//	res.get(i).canonicalise();
	//	}
	//	//remove the double results  
	//	
	//	for(int i = 0; i < k3; i++){
	//	for(int j = i +1; j < k3; j++){
	//	if (res.get(j).equals(res.get(i))) {res.remove(j); j = j -1;k3 = res.size();}
	//	}
	//	}
	//
	//	if(res.size() == 0){res.add(this);}
	//	//System.out.printf("\n %s \n", res.toString());
	//	return res;
	//      }


	//return upper and lower bound to string; assuming there is no more corner
	public String PrintLowerandUpper(){
		String s="";
		this.reduce();
		PTA pta = this.getPTA();
		int n = pta.numClocks + 1;
		for(int i = 1; i < n; i++){	
			if (this.d[0][i] != DB.INFTY){
				int val= DB.getAbsoluteDiff(this.d[0][i]);
				if (val == 0){
					s+=0+" , "+0;break;
				}
				else{
					s+=val+" , "+i;break;
				}
			}
		}
		for(int i = 1; i < n; i++){
			if (this.d[i][0] != DB.INFTY){
				s+= " , " + DB.getAbsoluteDiff(this.d[i][0])+" , "+i;break;
			}
		}
		return s;
	}





	//Returns wether this DBM is in the form i want(no corners)
	//public boolean isMySplitted(){
	//	  return (this.mySplit().size() == 1);
	//}

	/*	  int counter1 = 0;
    	  int counter2 = 0;
    	  for (int i = 0; i < pta.numClocks + 1; i++){
    		  if (this.d.[0].[i] < DB.INFTY){counter1++;}
    		  if (this.d.[i].[0] < DB.INFTY){counter2++;}
    	  }
    	  return (counter1 <= 1 & counter2 <= 1);
      }
	 */

	//set the constraint without verification (used for test)
	public void setConstraint(int x, int y, int db){
		d[x][y] = db;
	}

	//returns the bound from x where we integrate(the dbm has to be splitted)
	//the fst is the lowe, the snd is the upper
	// 	public ArrayList<int> getBounds (ArrayList<int> x){
	//		if (t.size() != pta.numclocks + 1) {return null;}
	//		if (!this.isMySplitted()){return null;}
	// 	while(d[0][i] == DB.INFTY)
	// 	{i++}
	// 	x[i] < d[0][i]

	//check wether x+t satifies the dbm

	//lower

	//}


	// Zone operations (modify the zone)




	// Methods required for Zone interface

	/**
	 * Get parent PTA
	 */
	public PTA getPTA()
	{
		return pta;
	}

	// Zone operations (modify the zone)

	/**
	 * Conjunction: add constraint x-y db
	 */
	public void addConstraint(int x, int y, int db)
	{
		// Check if this causes inconsistency (i.e. zone emptiness)
		// and, if so, flag this by setting d[0][0] to -1;
		if (DB.add(d[y][x], db) < DB.LEQ_ZERO)
			d[0][0] = DB.LEQ_MINUS_ONE;
		// Now add the constraint (if it is tighter than existing one)
		else if (db < d[x][y]) {
			int i, j, n, dTmp;
			// Store new constraint
			d[x][y] = db;
			// Partial re-canonicalisation
			// Note we do 2 (separate) outer iterations of Floyd-Warshall,
			// unlike the incorrect formulation in the DBM algorithm notes.
			n = pta.numClocks;
			for (i = 0; i < n + 1; i++) {
				for (j = 0; j < n + 1; j++) {
					dTmp = DB.add(d[i][x], d[x][j]);
					if (dTmp < d[i][j])
						d[i][j] = dTmp;
				}
			}
			for (i = 0; i < n + 1; i++) {
				for (j = 0; j < n + 1; j++) {
					dTmp = DB.add(d[i][y], d[y][j]);
					if (dTmp < d[i][j])
						d[i][j] = dTmp;
				}
			}
		}
	}

	/**
	 * Conjunction: add constraint c
	 */
	public void addConstraint(Constraint c)
	{
		addConstraint(c.x, c.y, c.db);
	}

	/**
	 * Conjunction: add multiple constraints
	 */
	public void addConstraints(Iterable<Constraint> constraints)
	{
		for (Constraint c : constraints)
			addConstraint(c);
	}

	/**
	 * Conjunction: with another zone
	 */
	public void intersect(Zone z)
	{
		int i, j, n;
		DBM dbm = (DBM) z;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				if (i != j && !DB.isInfty(dbm.d[i][j]))
					addConstraint(i, j, dbm.d[i][j]);
			}
		}
	}

	/**
	 * Up, i.e. let time elapse, subject to some constraints
	 */
	public void up(Iterable<Constraint> constraints)
	{
		int i, n;
		n = pta.numClocks;
		for (i = 1; i < n + 1; i++) {
			d[i][0] = DB.INFTY;
		}
		if (constraints != null) {
			for (Constraint c : constraints) {
				addConstraint(c);
			}
		}
	}

	/**
	 * Down, i.e. zone which can reach this one, subject to some constraints
	 */
	public void down(Iterable<Constraint> constraints)
	{
		int i, j, n;
		n = pta.numClocks;
		for (i = 1; i < n + 1; i++) {
			d[0][i] = DB.LEQ_ZERO;
			for (j = 1; j < n + 1; j++) {
				if (d[j][i] < d[0][i])
					d[0][i] = d[j][i];
			}
		}
		if (constraints != null) {
			for (Constraint c : constraints) {
				addConstraint(c);
			}
		}
	}

	/**
	 * Free, i.e. remove constraints on a clock
	 */
	public void free(int x)
	{
		int i, n;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			if (i != x) {
				d[x][i] = DB.INFTY;
				d[i][x] = d[i][0];
			}
		}
	}

	/**
	 * Reset clock x
	 */
	public void reset(int x, int v)
	{
		int i, n;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			d[x][i] = DB.add(DB.createLeq(v), d[0][i]);
			d[i][x] = DB.add(d[i][0], DB.createLeq(-v));
		}
	}

	/**
	 * Backward reset of clock x
	 */
	public void backReset(int x, int v)
	{
		// Conjunction with constraint x=v
		// (i.e. x-0 <= v && 0-x <= -v)
		addConstraint(x, 0, DB.createLeq(v));
		addConstraint(0, x, DB.createLeq(-v));
		// Then "free" clock x
		free(x);
	}

	/**
	 * c-Closure
	 */
	public void cClosure(int c)
	{
		int i, j, n;
		if (isEmpty())
			return;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				if (!DB.isInfty(d[i][j]) && DB.createLeq(c) < d[i][j]) {
					d[i][j] = DB.INFTY;
				} else if (!DB.isInfty(d[i][j]) && d[i][j] < DB.createLt(-c)) {
					d[i][j] = DB.createLt(-c);
				}
			}
		}
		canonicalise();
	}

	// Zone operations (create new zone)

	/**
	 * Complement this DBM; creates a new DBM list as result.
	 */
	public DBMList createComplement()
	{
		DBMList list = new DBMList(pta);
		DBM dbmNew;
		int i, j, n;
		// Special case: complement of empty DBM is True
		if (isEmpty()) {
			list.addDBM(createTrue(pta));
			return list;
		}
		n = d.length - 1;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				if (i == j)
					continue;
				if (DB.isInfty(d[i][j]))
					continue;
				dbmNew = (DBM) new DBMFactory().createTrue(pta);
				dbmNew.addConstraint(j, i, DB.dual(d[i][j]));
				if (!dbmNew.isEmpty()) {
					list.addDBM(dbmNew);
				}
			}
		}
		return list;
	}

	// Zone queries (do not modify the zone)

	/**
	 * Is this zone empty (i.e. inconsistent)?
	 */
	public boolean isEmpty()
	{
		// Internally, inconsistency is flagged by setting d[0][0] to -1.
		// (Note: strictly speaking "<0" checks that the difference bound is less than "<0".)
		return d[0][0] < 0;
	}

	/**
	 * Is constraint c satisfied by this zone (i.e does it overlap)?
	 */
	public boolean isSatisfied(Constraint c)
	{
		return DB.add(c.db, d[c.y][c.x]) > 0;
	}

	/**
	 * Is a DBM (fully) included in this zone?
	 */
	public boolean includes(DBM dbm)
	{
		int i, j, n;
		int[][] d2 = dbm.d;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				if (d[i][j] < d2[i][j])
					return false;
			}
		}
		return true;
	}

	public boolean strictLessDim(DBM dbm)
	{
		int i, j, n;
		int[][] d2 = dbm.d;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = i+1; j < n + 1; j++) {
				if (DB.compareBounds(d[i][j],d[j][i]) & !DB.compareBounds(d2[i][j],d2[j][i]))
					return true;
			}
		}
		return false;
	}

	/**
	 * Get the minimum value of a clock. 
	 */
	public int getClockMin(int x)
	{
		return -DB.getSignedDiff(d[0][x]);
	}

	/**
	 * Get the maximum value of a clock. 
	 */
	public int getClockMax(int x)
	{
		return DB.getSignedDiff(d[x][0]);
	}

	/**
	 * Check if a clock is unbounded (can be infinite).
	 */
	public boolean clockIsUnbounded(int x)
	{
		return DB.isInfty(d[x][0]);
	}

	/**
	 * Check if all clocks are unbounded (can be infinite).
	 */
	public boolean allClocksAreUnbounded()
	{
		int i, n;
		n = pta.numClocks;
		for (i = 1; i < n + 1; i++) {
			if (!DB.isInfty(d[i][0])) {
				return false;
			}
		}
		return true;
	}

	// Misc

	/**
	 * Clone this zone
	 */
	public DBM deepCopy()
	{
		int i, j, n;
		DBM copy = new DBM(pta);
		n = pta.numClocks;
		copy.d = new int[n + 1][n + 1];
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				copy.d[i][j] = d[i][j];
			}
		}
		return copy;
	}

	/**
	 * Get storage info string
	 */
	public String storageInfo()
	{
		return "DBM with " + pta.numClocks + " clocks";
	}

	// Standard Java methods

	public int hashCode()
	{
		// Simple hash code
		return pta.numClocks;
	}

	public boolean equals(Object o)
	{
		DBM dbm;
		int i, j, n;
		if (o == null)
			return false;
		try {
			dbm = (DBM) o;
		} catch (ClassCastException e) {
			return false;
		}
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {//This assumes canonical form
			for (j = 0; j < n + 1; j++) {
				if (d[i][j] != dbm.d[i][j])
					return false;
			}
		}
		return true;
	}

	// To string methods

	public String toString()
	{
		return toStringTextual();
	}

	/**
	 * Convert to string - textual representation
	 */
	public String toStringTextual()
	{
		int i, j, n;
		boolean first = true;
		String s = "", s2;
		n = pta.numClocks;
		// Trivial case - empty
		if (isEmpty())
			return "empty";
		// Generate textual description for each difference (pair)
		for (i = 0; i < n + 1; i++) {
			for (j = i + 1; j < n + 1; j++) {
				s2 = null;
				if (!DB.isInfty(d[i][j])) {
					if (!DB.isInfty(d[j][i])) {
						s2 = DB.constraintPairToString(i, j, d[i][j], d[j][i], pta);
					} else {
						s2 = DB.constraintToString(i, j, d[i][j], pta);
					}
				} else if (!DB.isInfty(d[j][i])) {
					s2 = DB.constraintToString(j, i, d[j][i], pta);
				}
				if (s2 != null) {
					if (!first)
						s += ",";
					else
						first = false;
					s += s2;
				}
			}
		}
		if ("".equals(s))
			return "true";
		else
			return "{" + s + "}";
	}

	/**
	 * Convert to string - DBM representation
	 */
	public String toStringDBM()
	{
		int i, j, n;
		String s = "[ ";
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				if (j > 0)
					s += " ";
				s += DB.toString(d[i][j]);
			}
			if (i < n)
				s += ", ";
		}
		s += " ]";
		return s;
	}

	public void printDBM()
	{
		int i, j, n;
		n = pta.numClocks+1;
		for (i = 0; i < n ; i++) {
			System.out.printf("\n");
			for (j = 0; j < n ; j++) {
				System.out.printf("%d ",d[i][j]);
			}
		}
	}



	/* Private utility methods */

	/**
	 * Canonicalise, by applying Floyd-Warshall SPP algorithm 
	 */
	public void canonicalise()//changed to public 
	{
		int k, i, j, db, n;
		n = pta.numClocks;
		for (k = 0; k < n + 1; k++) {
			for (i = 0; i < n + 1; i++) {
				for (j = 0; j < n + 1; j++) {
					db = DB.add(d[i][k], d[k][j]);
					if (db < d[i][j])
						d[i][j] = db;
				}
			}
		}
	}

	/* Static zone creation methods */

	/**
	 * All clocks = 0
	 */
	public static DBM createZero(PTA pta)
	{
		int i, j, n;
		DBM dbm = new DBM(pta);
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				dbm.d[i][j] = DB.LEQ_ZERO;
			}
		}
		return dbm;
	}

	/**
	 * All clocks any (non-negative) value
	 */
	public static DBM createTrue(PTA pta)
	{
		int i, j, n;
		DBM dbm = new DBM(pta);
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = 0; j < n + 1; j++) {
				if (i == j)
					dbm.d[i][j] = DB.LEQ_ZERO;
				else if (i == 0)
					dbm.d[i][j] = DB.LEQ_ZERO;
				else
					dbm.d[i][j] = DB.INFTY;
			}
		}
		return dbm;
	}

	/**
	 * Zone defined by set of constraints
	 */
	public static DBM createFromConstraints(PTA pta, Iterable<Constraint> constrs)
	{
		DBM dbm = createTrue(pta);
		for (Constraint c : constrs) {
			dbm.addConstraint(c);
		}
		return dbm;
	}


	/**
	 * Zone defined by set of constraints
	 */
	public static DBM createFromTransition(Transition tr)
	{
		DBM dbm = createTrue(tr.getParent());
		for (Constraint c : tr.guard) {
			dbm.addConstraint(c);
		}
		return dbm;
	}

	public Set<Constraint> toConstraint(){
		int n = pta.numClocks;
		Set<Constraint> res = new HashSet<Constraint>();
		for (int i = 0; i < n + 1; i++) {
			for (int j = 0; j < n + 1; j++) {
				//Constraint res_aux = new Constraint(i, j, d[i][j]);
				res.add(new Constraint(i, j, d[i][j]));		 
			}
		}
		return res;
	}

	//	public boolean yieldsPunctual(DBM M){
	//	DBM g=this.deepCopy();
	//	g.up();
	//	g.intersect(M);
	//	return (g.isEmpty() & g.isPunctual());	
	//	}

	//The following is to be used for guard only.
	public boolean isPunctual(){
		this.canonicalise();
		int n = pta.numClocks + 1;
		for(int i = 1; i < n; i++){
			if(d[i][0]<=1 | DB.compareBounds(d[i][0],d[0][i])){ 
				return true;} 
			for(int j =i+1 ; j < n; j++){
				if(!DB.isInfty(d[i][j]) & DB.compareBounds(d[i][j],d[j][i])& (DB.getSignedDiff(d[i][j])!=0)){
					return true;}
			}
		}
		return false;
	}

	//not used
	//	public void closeIt(){
	//		int i, j, n;
	//		n = pta.numClocks;
	//		for (i = 0; i < n + 1; i++) {
	//			for (j = 0; j < n + 1; j++) {
	//				if (i != j)
	//					d[i][j]=DB.closeIt(d[i][j]);
	//			}
	//		}	
	//	}

	//	//EQUALITY NOT TREATED
	public boolean openIt(Zone z){
		DBM m= (DBM) z;
		m.canonicalise();//TODO remove this if already canonical
		int i, j, n;
		n = pta.numClocks;
		for (i = 0; i < n + 1; i++) {
			for (j = i+1; j < n + 1; j++) {
				if (!DB.compareBounds(m.d[i][j], m.d[j][i]))//We keep equality, and open non punctual intervals.
				{d[i][j]=DB.openIt(d[i][j]);
				d[j][i]=DB.openIt(d[j][i]);
				}
			}
			this.canonicalise();//to see whether the zone is empty or not.
		}
		return(this.isEmpty());
	}

	/***
	 * 
	 * @return a list of list of index of clocks which are pairwise equals (the fist list is the clocks equals to 0) and the first element of each list is the smallest index.
	 */
	public ArrayList<ArrayList<Integer>> reducedCoordinate(){
		ArrayList<ArrayList<Integer>> res= new ArrayList<ArrayList<Integer>>();
		//System.out.printf("%s",res.toString());
		int i, j, n;
		n = pta.numClocks;
		int[] rC =new int [n+1];
		ArrayList<Integer> zeroList=new ArrayList<Integer>();
		//System.out.printf("%s",zeroList.toString());
		zeroList.add(0);
		//System.out.printf("%s",zeroList.toString());
		for (i = 1; i < n + 1; i++) {
			if (d[i][0]<2)
			{rC[i]=0;zeroList.add(i);}
			else
			{rC[i]=i;}
		}
		//System.out.printf("%s",zeroList.toString());
		int numParts=0;//the number of parts
		res.add(zeroList);
		//System.out.printf("%s",res.toString());
		for (i = 1; i < n + 1; i++) {
			if (rC[i]!=i) {continue;}//This clocks is already equals to another, continue!
			numParts++;
			rC[i]=numParts;
			ArrayList<Integer> newList=new ArrayList<Integer>();
			newList.add(i);
			res.add(newList);
			for (j = i+1; j < n + 1; j++) {
				if (DB.compareBounds(d[i][j], d[j][i]))
				{rC[j]=rC[i];res.get(numParts).add(j);}
			}
		}
		//System.out.printf("\n%s %s \n",this.toString(),res.toString());
		return(res);
	}
	
}
